package com.systemj;

import android.app.TabActivity;
import android.widget.TabHost;
import android.os.Bundle;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.Context;
import android.content.BroadcastReceiver;
import android.content.ServiceConnection;
import android.content.ComponentName;
import android.util.Log;
import android.os.IBinder;
import android.os.Handler;
import android.os.Message;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.widget.EditText;
import android.app.ProgressDialog;
import java.util.Set;

public class BluetoothActivity extends TabActivity
{
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        Log.i("Bluetooth Test", "Starting");
        Android.setBluetoothActivity(this);
        
        progress_dialog = ProgressDialog.show(BluetoothActivity.this, "", "Connecting to Server...", true);
        
        tabhost = getTabHost();
        Intent i1 = new Intent().setClass(this, TasksActivity.class);
        Intent i2 = new Intent().setClass(this, ScannerActivity.class);
        tabhost.addTab(tabhost.newTabSpec("tab_tasks").setIndicator("Task List").setContent(i1));                
        tabhost.addTab(tabhost.newTabSpec("tab_scanner").setIndicator("Scanner").setContent(i2));

        tabhost.setCurrentTab(0);
        
        adapter = BluetoothAdapter.getDefaultAdapter();
        if(adapter == null)
        {
        	// no bluetooth, panic?
        	Log.i("Bluetooth Test", "Bluetooth not supported");
        	return;
        }
        
        // register broadcast receiver
        IntentFilter filter = new IntentFilter(BluetoothDevice.ACTION_FOUND);
    	registerReceiver(receiver, filter);
        
        // try to enable bluetooth
        if(!adapter.isEnabled())
        {
        	Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
        	startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
        	Log.i("Bluetooth Test", "Sent enable request");
        }
        else
        {
        	Log.i("Bluetooth Test", "Bluetooth already enabled");
        	findServerDevice();
        }
    }
    
    Handler handler = new Handler()
    {
    	public void handleMessage(Message msg)
		{
			Bundle data = msg.getData();
			int op = data.getInt("op");
			AlertDialog.Builder builder = new AlertDialog.Builder(BluetoothActivity.this);
			if(alert != null)
			{
				alert.dismiss();
			}
			switch(op)
			{
				case SHOW_ERROR:
					String message = data.getString("message");
					final boolean fatal = data.getBoolean("fatal");
					builder.setTitle("Server Error").setMessage(message)
					.setCancelable(false).setPositiveButton("OK", new DialogInterface.OnClickListener()
					{
						public void onClick(DialogInterface dialog, int id)
						{
							dialog.dismiss();
							if(fatal)
							{
								Intent intent = new Intent().setClass(BluetoothActivity.this, RestartActivity.class);
        						startActivity(intent);
                				finish();
							}
						}
					});
					alert = builder.create();
					alert.show();
					break;
				case SHOW_LOGIN:
					final EditText input = new EditText(BluetoothActivity.this);  
					builder.setTitle("Login").setMessage("Please enter your name:").setView(input)
					.setCancelable(false).setPositiveButton("Login", new DialogInterface.OnClickListener()
					{
						public void onClick(DialogInterface dialog, int id)
						{
							String username = input.getText().toString();  
							dialog.dismiss();
				
							// send the username to systemj
							systemjservice.handleEvent("login", username);
						}
					});
					alert = builder.create();
					progress_dialog.dismiss();
					alert.show();
					break;
			}
		}
    };
    
    public void showError(String message, boolean fatal)
    {
		Message msg = handler.obtainMessage();
    	Bundle b = new Bundle();
    	b.putInt("op", SHOW_ERROR);
    	b.putString("message", message);
    	b.putBoolean("fatal", fatal);
    	msg.setData(b);
    	msg.sendToTarget();
	}
	
	protected void showLogin()
	{
		Message msg = handler.obtainMessage();
    	Bundle b = new Bundle();
    	b.putInt("op", SHOW_LOGIN);
    	msg.setData(b);
    	msg.sendToTarget();
	}
    
    protected void onActivityResult(int requestCode, int resultCode, Intent data)
    {
    	switch(requestCode)
    	{
    		case REQUEST_ENABLE_BT:
				switch(resultCode)
				{
					case RESULT_OK:
						Log.i("Bluetooth Test", "Bluetooth enabled");
						findServerDevice();
						break;
					case RESULT_CANCELED:
						Log.i("Bluetooth Test", "Bluetooth not enabled");
						break;
				}
				break;
		}
    }
    
    protected void findServerDevice()
    {
    	// lets see if the device is already paired
    	Set<BluetoothDevice> paired = adapter.getBondedDevices();
    	if(paired.size() > 0)
    	{
    		for(BluetoothDevice device : paired)
    		{
    			Log.i("Bluetooth Test", "Found paired device: " + device.getName());
    			if(device.getName().equals(server_name))
    			{
    				server = device;
    				Log.i("Bluetooth Test", "Server set to device: " + server.getName());
    				bindService(new Intent(BluetoothActivity.this, SystemJService.class), connection, Context.BIND_AUTO_CREATE);
    				adapter.cancelDiscovery();
    				connect = new BluetoothConnectThread(server, systemjservice);
    				connect.start();
    				return;
    			}
    		}
    	}
    	
    	// otherwise we scan for devices and look for it
    	// this is asyncronous and the results are received by the broadcast receiver below
    	adapter.startDiscovery();
    }
    
    private final BroadcastReceiver receiver = new BroadcastReceiver()
    {
    	public void onReceive(Context context, Intent intent)
    	{
    		String action = intent.getAction();
    		if(BluetoothDevice.ACTION_FOUND.equals(action))
    		{
    			BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
    			Log.i("Bluetooth Test", "Discovered device: " + device.getName());
    			if(device.getName().equals(server_name))
    			{
    				server = device;
    				Log.i("Bluetooth Test", "Server set to device: " + server.getName());
    				adapter.cancelDiscovery();
    				bindService(new Intent(BluetoothActivity.this, SystemJService.class), connection, Context.BIND_AUTO_CREATE);
    				
    				connect = new BluetoothConnectThread(server, systemjservice);
    				connect.start();
    			}
    		}
    	}
    };
    
    @Override
    protected void onResume()
    {
    	super.onResume();
    	Android.setSystemJService(systemjservice);
    }
    
    @Override
    protected void onDestroy()
    {
    	super.onDestroy();
    	Log.i("Bluetooth Test", "destroy");
    	unregisterReceiver(receiver);
    	unbindService(connection);
    	adapter.cancelDiscovery();
    	connect.cancel();
    } 
    
    private ServiceConnection connection = new ServiceConnection()
    {
		public void onServiceConnected(ComponentName className, IBinder service) 
		{
		    systemjservice = ((SystemJService.SystemJBinder)service).getService();
		}

		public void onServiceDisconnected(ComponentName className) 
		{
		    systemjservice = null;
		}
	};
	
	private SystemJService systemjservice;
	    
    private BluetoothAdapter adapter;
    private BluetoothDevice server;
    private BluetoothConnectThread connect;
    private ProgressDialog progress_dialog;
    private AlertDialog alert = null;
        
    private static String server_name = "THEBEAST";
    private static final int REQUEST_ENABLE_BT = 0;
    
    private TabHost tabhost;
    
    private static final int SHOW_ERROR = 0;
    private static final int SHOW_LOGIN = 1;
}
