package com.systemj;

import java.util.ArrayList;
import java.util.HashMap;

import systemj.interfaces.GenericSignalReciever;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;

public class EventReciever implements GenericSignalReciever
{
	public EventReciever()
	{
		looper = new LooperThread();
		looper.start();
	}

	public void configure(HashMap<String, String> data) throws RuntimeException
	{
		// main configuration data we need is the event name
		if(data.containsKey("EventName"))
		{
			String name = data.get("EventName");
			if(data.containsKey("ObjectID"))
			{
				String s = data.get("ObjectID");
				try
				{
					objectid = R.id.class.getDeclaredField(s).getInt(null);
				}
				catch (IllegalAccessException e)
				{
					throw new RuntimeException("Unable to access: " + s);
				}
				catch (NoSuchFieldException e)
				{
					throw new RuntimeException("Unknown id of event reciever source: " + s);
				}
				Android.getSystemJService().setEventHandler(name + ":" + objectid, this);
			}
			else
			{
				Android.getSystemJService().setEventHandler(name, this);
			}
			// also need to see if there is a timeout specified
			if(data.containsKey("TimeOutValue"))
			{
				timeout = Integer.parseInt(data.get("TimeOutValue"));
			}
			else
			{
				timeout = 10; // default to 100ms = 1/10sec
			}
		}
		else
		{
			throw new RuntimeException("The configuration parameter 'EventName' is required!");
		}
	}

	public Object getBuffer()
	{
		return buffer;
	}

	public boolean getDone()
	{
		return done;
	}

	public int getTimeOut()
	{
		return timeout;
	}
	
	public Handler getHandler()
	{
		return looper.h;
	}

	public void run()
	{
		try
		{
			synchronized(looper)
			{
				looper.wait(timeout);
			}
			if(!done)
			{
				ArrayList<Object> l = new ArrayList<Object>();
				l.add(new Boolean(false));
				setBuffer(l);
				setDone(true);
			}
		}
		catch (InterruptedException e)
		{
			Log.e("SystemJRuntime", "Thread wait interrupted.", e);
		}
	}

	public synchronized void setBuffer(Object buffer)
	{
		this.buffer = buffer;
	}

	public synchronized void setDone(boolean done)
	{
		this.done = done;
	}
	
	private class LooperThread extends Thread
	{
		public void run()
		{
			Looper.prepare();
			
			h = new Handler()
			{
				public void handleMessage(Message msg)
				{
					ArrayList<Object> l = new ArrayList<Object>();
					l.add(new Boolean(true));
					l.add(msg.obj);
					
					setBuffer(l);
					setDone(true);
					
					synchronized(looper)
					{
						looper.notify();
					}
				}
			};
			
			Looper.loop();
		}
		
		public Handler h;
	}

	private Object buffer;
	private boolean done;
	private int timeout;
	private int objectid = -1;
	
	private LooperThread looper;
	private AndroidIOLogger Log = new AndroidIOLogger();
}
