package com.systemj;

import android.app.Activity;
import android.os.Bundle;
import android.content.Intent;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.os.Handler;
import android.os.Message;
import android.app.ProgressDialog;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.util.Log;
import java.util.HashMap;
import java.util.Iterator;
import java.util.ArrayList;
import org.json.JSONObject;
import org.json.JSONException;

public class ScannerActivity extends Activity 
{
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.scanner);
        
        Android.setScannerActivity(this);
        
        View scanner_button = findViewById(R.id.scanner);
        scanner_button.setOnClickListener(scanner_button_listener);
        View stocktake_button = findViewById(R.id.stocktake);
        stocktake_button.setOnClickListener(stocktake_button_listener);
        
        View stocktake_scan_button = findViewById(R.id.stocktake_scan);
        stocktake_scan_button.setVisibility(View.INVISIBLE);
        stocktake_scan_button.setOnClickListener(stocktake_scan_listener);
    }
    
    private final View.OnClickListener scanner_button_listener = new View.OnClickListener()
    {
    	public void onClick(View v)
    	{
    		Intent intent = new Intent("com.google.zxing.client.android.SCAN");
        	intent.putExtra("SCAN_MODE", "CODE_39_MODE");
        	startActivityForResult(intent, REQUEST_SCAN);
    	}
    };
    
    private final View.OnClickListener stocktake_button_listener = new View.OnClickListener()
    {
    	public void onClick(View v)
    	{
    		Button b = (Button)v;
    		if(stocktake_active)
    		{
    			progress = ProgressDialog.show(ScannerActivity.this, "", "Sending stocktake data to server...", true);
    		
    			b.setText("Start Stocktake");
    			findViewById(R.id.stocktake_scan).setVisibility(View.INVISIBLE);
    			
    			ArrayList<JSONObject> stocks = new ArrayList<JSONObject>();
    			Iterator it = stocktake_data.keySet().iterator();
    			while(it.hasNext())
    			{
    				int key = (Integer)it.next();
    				int val = stocktake_data.get(key);
    				
    				try
    				{
						JSONObject data = new JSONObject();
						data.put("barcode", key);
						data.put("amount", val);
						stocks.add(data);
					}
					catch(JSONException e)
					{
						Log.e("Bluetooth Test", "JSON Error", e);
					}
    			}
    			Android.getSystemJService().handleEvent("stocktakeIn", stocks);
    			stocktake_data.clear();
    		}
    		else
    		{
    			findViewById(R.id.stocktake_scan).setVisibility(View.VISIBLE);
    			b.setText("Finish Stocktake");
    		}
    		stocktake_active = !stocktake_active;
    	}
    };
    
    private final View.OnClickListener stocktake_scan_listener = new View.OnClickListener()
    {
    	public void onClick(View v)
    	{
    		Intent intent = new Intent("com.google.zxing.client.android.SCAN");
        	intent.putExtra("SCAN_MODE", "CODE_39_MODE");
        	startActivityForResult(intent, STOCKTAKE_SCAN);
    	}
    };
    
    public void onActivityResult(int requestCode, int resultCode, Intent intent)
    {
		if(requestCode == REQUEST_SCAN)
		{
		    switch(resultCode)
		    {
				case RESULT_OK:
				    String contents = intent.getStringExtra("SCAN_RESULT");
				    Log.i("Scanner", "Contents: " + contents);
				    // send scan result to server
				    progress = ProgressDialog.show(ScannerActivity.this, "", "Sending " + contents + " to server...", true);
				    
				    ServiceWaitThread s = new ServiceWaitThread("barcodeIn", new Integer(contents));
				    //showPriceResult("A product", 39.99, 5); // this will be called from systemj
				    break;
				case RESULT_CANCELED:
				    // Handle cancel
				    break;
		    }
		}
		else
		{
			switch(resultCode)
		    {
				case RESULT_OK:
				    int barcode = new Integer(intent.getStringExtra("SCAN_RESULT"));
				    if(stocktake_data.containsKey(barcode))
				    {
				    	int l = stocktake_data.get(barcode);
				    	l++;
				    	stocktake_data.remove(barcode);
				    	stocktake_data.put(barcode, l);
				    }
				    else
				    {
				    	stocktake_data.put(barcode, 1);
				    }
				    break;
				case RESULT_CANCELED:
				    // Handle cancel
				    break;
		    }
		}
	}
	
	public final Handler handler = new Handler()
    {
		public void handleMessage(Message msg)
		{
			progress.dismiss();
			Bundle data = msg.getData();
			if(data.containsKey("name"))
			{
				AlertDialog.Builder builder = new AlertDialog.Builder(ScannerActivity.this);
				String text = "Product Name: " + data.getString("name") + "\nPrice: $" + data.getDouble("price") + "\nStock: " + data.getInt("stock");
				builder.setTitle("Product Details").setMessage(text).setCancelable(false).setPositiveButton("OK", new DialogInterface.OnClickListener()
				{
					public void onClick(DialogInterface dialog, int id)
					{
						dialog.cancel();
					}
				});
				AlertDialog alert = builder.create();
				alert.show();
			}
		}
	};
	
	public void showPriceResult(String name, double price, int stock)
	{
		Message msg = handler.obtainMessage();
		Bundle b = new Bundle();
		b.putString("name", name);
		b.putDouble("price", price);
		b.putInt("stock", stock);
		msg.setData(b);
		handler.sendMessage(msg);
	}
	
	public void dismissProgress()
	{
		Message msg = handler.obtainMessage();
		Bundle b = new Bundle();
		msg.setData(b);
		handler.sendMessage(msg);
	}				
	
	public static final int REQUEST_SCAN = 0;
	public static final int STOCKTAKE_SCAN = 1;
	
	private HashMap<Integer,Integer> stocktake_data = new HashMap<Integer,Integer>();
	private boolean stocktake_active = false;
	private ProgressDialog progress;
}
