package com.systemj;

import systemj.bootstrap.SystemJProgram;
import systemj.interfaces.IOLogger;

import android.content.pm.ServiceInfo;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.app.Service;
import android.os.Binder;
import android.os.IBinder;
import android.content.Intent;
import android.content.ComponentName;
import android.app.Notification;
import android.app.NotificationManager;
import android.content.Context;
import android.app.PendingIntent;
import android.net.Uri;

import java.util.HashMap;
import java.util.ArrayList;

public class SystemJService extends Service
{
    public class SystemJBinder extends Binder 
    {
        public SystemJService getService()
        {
            return SystemJService.this;
        }
    }

    @Override
    public void onCreate() 
    {
    	Android.setSystemJService(this);
    	Log = new AndroidIOLogger();
        try
		{
			ServiceInfo ai = getPackageManager().getServiceInfo(new ComponentName(this, SystemJService.class), PackageManager.GET_META_DATA);
			Bundle bundle = ai.metaData;
		    sysjname = bundle.getString("systemj.program");
		    xmlfile = bundle.getInt("systemj.xmlfile");
		    Log.i("SystemJRuntime", "Loading class name: " + sysjname);
		    Log.i("SystemJRuntime", "Loading xml file at: " + xmlfile);
		}
		catch (Exception e)
		{
			Log.e("SystemJRuntime", "Error retrieving meta data tags.", e);
		}

		try
		{
			if(sysjname != null)
			{
				sysj = (SystemJProgram) Class.forName(sysjname).newInstance();
				sysj.setIOLogger(Log);
				if(xmlfile != -1)
				{
					AndroidParser parser = new AndroidParser();
					parser.setTarget(xmlfile, getApplicationContext());
					sysj.setParser(parser);
				}
				
				for(int i = 0; i < signames.size(); i++)
				{
					sysj.bindGenericSignal(signames.get(i), sigdata.get(i));
				}
				
				sysj.start();
				
				synchronized(sysj)
				{
					sysj.wait();
				}
			}
			else
			{
				Log.w("SystemJRuntime", "No SystemJ class specified, skipping SystemJ bootstrap.");
			}
		}
		catch (Exception e)
		{
			Log.e("SystemJRuntime", "Error bootstraping SystemJ", e);
		}
		handleEvent("onCreate");
    }

    @Override
    public void onDestroy()
    {
        handleEvent("onDestroy");
    }

    @Override
    public IBinder onBind(Intent intent)
    {
        return binder;
    }
    
    public void setSystemJClassName(String n)
	{
		sysjname = n;
	}
	
	public void setXMLReference(int r)
	{
		xmlfile = r;
	}
	
	public void setEventHandler(String e, EventReciever r)
	{
		eventrecievers.put(e, r);
	}
	
	public void setSignal(String name, HashMap<String, String> data)
	{
		signames.add(name);
		sigdata.add(data);
	}
	
	public void handleEvent(String eventname)
	{
		if(eventrecievers.containsKey(eventname))
		{
			Handler h = eventrecievers.get(eventname).getHandler();
			Message msg = Message.obtain(h);
			msg.sendToTarget();
		}
	}
	
	public void handleEvent(String eventname, Object data)
	{
		if(eventrecievers.containsKey(eventname))
		{
			Handler h = eventrecievers.get(eventname).getHandler();
			Message msg = Message.obtain(h);
			msg.obj = data;
			msg.sendToTarget();
		}
	}
	
	public void setTaskActivity(TasksActivity a)
	{
		tasks = a;
	}
	
	public TasksActivity getTaskActivity()
	{
		return tasks;
	}
	
	public void notifyUser(CharSequence title, CharSequence ticker, CharSequence content)
	{
		int icon = R.drawable.notification;
		long when = System.currentTimeMillis();
		Context context = getApplicationContext();
		
		Intent i = new Intent(this, BluetoothActivity.class);
		PendingIntent pi = PendingIntent.getActivity(this, 0, i, 0);

		Notification notification = new Notification(icon, ticker, when);
		notification.setLatestEventInfo(context, title, content, pi);
		
		notification.defaults |= Notification.DEFAULT_ALL;
		notification.flags |= Notification.FLAG_AUTO_CANCEL;
		
		String ns = Context.NOTIFICATION_SERVICE;
		NotificationManager manager = (NotificationManager)getSystemService(ns);
		
		manager.notify(NOTIFICATION_ID, notification);
	}

	protected SystemJProgram sysj;
	protected String sysjname = null;
	protected int xmlfile = -1;
	
	private TasksActivity tasks;
	
	private ArrayList<String> signames = new ArrayList<String>();
	private ArrayList<HashMap<String,String>> sigdata = new ArrayList<HashMap<String,String>>();

	private HashMap<String,EventReciever> eventrecievers = new HashMap<String,EventReciever>();
	private IOLogger Log;// = new AndroidIOLogger();

    private final IBinder binder = new SystemJBinder();
    
    private static final int NOTIFICATION_ID = 1;
}
