package com.systemj;

import android.app.ListActivity;
import android.os.Bundle;
import android.content.Intent;
import android.widget.ListAdapter;
import android.widget.ArrayAdapter;
import android.widget.AdapterView;
import android.widget.ListView;
import android.view.View;
import android.util.Log;
import java.util.ArrayList;
import android.os.Handler;
import android.os.Message;
import android.app.ProgressDialog;
import android.app.AlertDialog;
import android.content.DialogInterface;

public class TasksActivity extends ListActivity 
{
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.tasks);
        
        Android.setTasksActivity(this);
        
        adapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_multiple_choice);
        /*adapter.add("Do stocktake of Widget X");
        ids.add(1);
        adapter.add("Cleanup in aisle 7!");
        ids.add(2);*/
        
        setListAdapter(adapter);
        
        findViewById(R.id.bottom_control_bar).setVisibility(View.INVISIBLE);
        findViewById(R.id.mark_selected_button).setOnClickListener(mark_selected_listener);
        
        getListView().setOnItemLongClickListener(long_click_listener);
    }
    
    public void putTask(int id, String name, String desc)
    {
    	Message msg = handler.obtainMessage();
    	Bundle b = new Bundle();
    	b.putInt("op", OP_PUT_TASK);
    	b.putInt("id", id);
    	b.putString("title", name);
    	b.putString("desc", desc);
    	msg.setData(b);
    	msg.sendToTarget();
    }
    
    public void removeTask(int id)
    {
    	Message msg = handler.obtainMessage();
    	Bundle b = new Bundle();
    	b.putInt("op", OP_REMOVE_TASK);
    	b.putInt("id", id);
    	msg.setData(b);
    	msg.sendToTarget();
    }
    
    public final Handler handler = new Handler()
    {
		public void handleMessage(Message msg)
		{
			Bundle data = msg.getData();
			int op = data.getInt("op");
			switch(op)
			{
				case OP_PUT_TASK:
					ids.add(data.getInt("id"));
					Log.i("Tasks", data.getString("title"));
					adapter.add(data.getString("title"));
					descs.add(data.getString("desc"));
					break;
				case OP_REMOVE_TASK:
					int id = data.getInt("id");
					int i = ids.indexOf(id);
					String item = (String)adapter.getItem(i);
					adapter.remove(item);
					ids.remove(i);
					completed_count--;
					if(completed_count <= 0)
					{
						completed_count = 0;
						progress_dialog.dismiss();
					}
					break;
			}
		}
	};
    
    @Override
    public void onListItemClick(ListView l, View v, int position, long id)
    {
    	if(l.isItemChecked(position))
    	{
    		l.setItemChecked(position, true);
    		selected_count++;
    		if(selected_count == 1)
    		{
    			findViewById(R.id.bottom_control_bar).setVisibility(View.VISIBLE);
    		}
    	}
    	else
    	{
    		l.setItemChecked(position, false);
    		selected_count--;
    		if(selected_count == 0)
    		{
    			findViewById(R.id.bottom_control_bar).setVisibility(View.INVISIBLE);
    		}
    	}
    }
    
    private final View.OnClickListener mark_selected_listener = new View.OnClickListener()
    {
    	public void onClick(View v)
    	{
    		ListView list = (ListView)findViewById(android.R.id.list);
    		ArrayList<Integer> marked_ids = new ArrayList<Integer>();
    		for(int i = 0; i < adapter.getCount(); i++)
    		{
    			if(list.isItemChecked(i))
    			{
					marked_ids.add(ids.get(i));
					completed_count++;
    			}
    		}
			Android.getSystemJService().handleEvent("taskChanged", marked_ids);
    		list.clearChoices();
    		findViewById(R.id.bottom_control_bar).setVisibility(View.INVISIBLE);
    		selected_count = 0;
    		progress_dialog = ProgressDialog.show(TasksActivity.this, "", "Sending task statuses...", true);
    	}
    };
    
   	private final AdapterView.OnItemLongClickListener long_click_listener = new AdapterView.OnItemLongClickListener()
   	{
   		@Override
		public boolean onItemLongClick(AdapterView<?> av, View v, int pos, long id)
		{
    		AlertDialog.Builder builder = new AlertDialog.Builder(TasksActivity.this);
			builder.setTitle("Task Description").setMessage(descs.get(pos)).setCancelable(false).setPositiveButton("OK", new DialogInterface.OnClickListener()
			{
				public void onClick(DialogInterface dialog, int id)
				{
					dialog.dismiss();
				}
			});
			AlertDialog alert = builder.create();
			alert.show();
			
			return false;
    	}
    };
    
    public void onPause()
    {
    	super.onPause();
    	selected_count = 0;
    	ListView list = (ListView)findViewById(android.R.id.list);
    	list.clearChoices();
    }
    
    private int completed_count = 0;
    private int selected_count = 0;
    private ArrayAdapter adapter;
    private ArrayList<Integer> ids = new ArrayList<Integer>();
    private ArrayList<String> descs = new ArrayList<String>();
    private ProgressDialog progress_dialog;
    
    private static final int OP_PUT_TASK = 0;
    private static final int OP_REMOVE_TASK = 1;
}
